@@ -55,10 +55,14 @@ public:
     PendingOperation *requestDisconnect();
 
     SimpleStatusSpecMap allowedPresenceStatuses() const;
+    uint maxPresenceStatusMessageLength() const;
+
     PendingOperation *setSelfPresence(const QString &status, const QString &statusMessage);
 
     PendingChannel *createChannel(const QVariantMap &request);
+    PendingChannel *createChannel(const QVariantMap &request, int timeout);
     PendingChannel *ensureChannel(const QVariantMap &request);
+    PendingChannel *ensureChannel(const QVariantMap &request, int timeout);
 
     PendingHandles *requestHandles(HandleType handleType, const QStringList &names);
     PendingHandles *referenceHandles(HandleType handleType, const UIntList &handles);
