@@ -1040,7 +1040,7 @@ Connection::~Connection()
  * situations where objects constructed at different times by the account would have unpredictably
  * different construction settings (eg. subclass).
  *
- * \return Read-only pointer to the factory.
+ * \return A read-only pointer to the ChannelFactory object.
  */
 ChannelFactoryConstPtr Connection::channelFactory() const
 {
@@ -1055,7 +1055,7 @@ ChannelFactoryConstPtr Connection::channelFactory() const
  * situations where objects constructed at different times by the account would have unpredictably
  * different construction settings (eg. subclass).
  *
- * \return Read-only pointer to the factory.
+ * \return A read-only pointer to the ContactFactory object.
  */
 ContactFactoryConstPtr Connection::contactFactory() const
 {
@@ -1065,7 +1065,7 @@ ContactFactoryConstPtr Connection::contactFactory() const
 /**
  * Return the connection manager name of this connection.
  *
- * \return The connection manager name of this connection.
+ * \return The connection manager name.
  */
 QString Connection::cmName() const
 {
@@ -1075,7 +1075,7 @@ QString Connection::cmName() const
 /**
  * Return the protocol name of this connection.
  *
- * \return The protocol name of this connection.
+ * \return The protocol name.
  */
 QString Connection::protocolName() const
 {
@@ -1087,9 +1087,9 @@ QString Connection::protocolName() const
  *
  * Change notification is via the statusChanged() signal.
  *
- * This method requires Connection::FeatureCore to be enabled.
+ * This method requires Connection::FeatureCore to be ready.
  *
- * \return The status of this connection, as defined in ConnectionStatus.
+ * \return The status as #ConnectionStatus.
  * \sa statusChanged(), statusReason(), errorDetails()
  */
 ConnectionStatus Connection::status() const
@@ -1106,10 +1106,10 @@ ConnectionStatus Connection::status() const
  * doesn't understand an error name given as the invalidation reason, which may in some cases be
  * domain/UI-specific.
  *
- * This method requires Connection::FeatureCore to be enabled.
+ * This method requires Connection::FeatureCore to be ready.
  *
+ * \return The status reason as #ConnectionStatusReason.
  * \sa invalidated(), invalidationReason()
- * \return The reason, as defined in ConnectionStatusReason.
  */
 ConnectionStatusReason Connection::statusReason() const
 {
@@ -1195,7 +1195,7 @@ Connection::ErrorDetails &Connection::ErrorDetails::operator=(
  *
  * Return whether or not the details are valid (have actually been received from the service).
  *
- * \return Validity of the details.
+ * \return \c true if valid, \c false otherwise.
  */
 
 /**
@@ -1209,7 +1209,7 @@ Connection::ErrorDetails &Connection::ErrorDetails::operator=(
  * The debug message is purely informational, offered for display for bug reporting purposes, and
  * should not be attempted to be parsed.
  *
- * \return Whether there is a debug message or not.
+ * \return \c true if debug message is present, \c false otherwise.
  * \sa debugMessage()
  */
 
@@ -1233,7 +1233,7 @@ Connection::ErrorDetails &Connection::ErrorDetails::operator=(
  *
  * This is useful for accessing domain-specific additional details.
  *
- * \return A map containing all details given in the low-level ConnectionError signal.
+ * \return The details of the connection error as QVariantMap.
  */
 QVariantMap Connection::ErrorDetails::allDetails() const
 {
@@ -1252,7 +1252,7 @@ QVariantMap Connection::ErrorDetails::allDetails() const
  * if the client doesn't understand what a particular value returned by invalidationReason() means,
  * as it may be domain-specific with some services.
  *
- * \return The error details.
+ * \return The error details as a Connection::ErrorDetails object.
  * \sa status(), statusReason(), invalidationReason()
  */
 const Connection::ErrorDetails &Connection::errorDetails() const
@@ -1272,9 +1272,9 @@ const Connection::ErrorDetails &Connection::errorDetails() const
  *
  * Change notification is via the selfHandleChanged() signal.
  *
- * This method requires Connection::FeatureCore to be enabled.
+ * This method requires Connection::FeatureCore to be ready.
  *
- * \return The handle representing the user on this connection.
+ * \return The user handle.
  * \sa selfHandleChanged(), selfContact()
  */
 uint Connection::selfHandle() const
@@ -1289,10 +1289,9 @@ uint Connection::selfHandle() const
  * Connection spends in status ConnectionStatusConnecting,
  * again staying fixed for the entire time in ConnectionStatusConnected.
  *
- * This method requires Connection::FeatureSimplePresence to be enabled.
+ * This method requires Connection::FeatureSimplePresence to be ready.
  *
- * \return A dictionary from string identifiers to structs for each valid
- *         status.
+ * \return The allowed statuses as a map from string identifiers to SimpleStatusSpec objects.
  */
 SimpleStatusSpecMap ConnectionLowlevel::allowedPresenceStatuses() const
 {
@@ -1314,15 +1313,15 @@ SimpleStatusSpecMap ConnectionLowlevel::allowedPresenceStatuses() const
 }
 
 /**
- * Returns the maximum length for a presence status message.
+ * Return the maximum length for a presence status message.
  *
  * The value may have changed arbitrarily during the time the
  * Connection spends in status ConnectionStatusConnecting,
  * again staying fixed for the entire time in ConnectionStatusConnected.
  *
- * This method requires Connection::FeatureSimplePresence to be enabled.
+ * This method requires Connection::FeatureSimplePresence to be ready.
  *
- * \return The maximum length for a presence status message, or 0 if there is no limit.
+ * \return The maximum length, or 0 if there is no limit.
  */
 uint ConnectionLowlevel::maxPresenceStatusMessageLength() const
 {
@@ -1360,7 +1359,6 @@ uint ConnectionLowlevel::maxPresenceStatusMessageLength() const
  * \param statusMessage The desired status message.
  * \return A PendingOperation which will emit PendingOperation::finished
  *         when the call has finished.
- *
  * \sa allowedPresenceStatuses()
  */
 PendingOperation *ConnectionLowlevel::setSelfPresence(const QString &status,
@@ -1393,9 +1391,9 @@ PendingOperation *ConnectionLowlevel::setSelfPresence(const QString &status,
  *
  * Change notification is via the selfContactChanged() signal.
  *
- * This method requires Connection::FeatureSelfContact to be enabled.
+ * This method requires Connection::FeatureSelfContact to be ready.
  *
- * \return The object representing the user on this connection.
+ * \return A pointer to the Contact object, or a null ContactPtr if unknown.
  * \sa selfContactChanged(), selfHandle()
  */
 ContactPtr Connection::selfContact() const
@@ -1416,9 +1414,9 @@ ContactPtr Connection::selfContact() const
  *
  * Change notification is via the accountBalanceChanged() signal.
  *
- * This method requires Connection::FeatureAccountBalance to be enabled.
+ * This method requires Connection::FeatureAccountBalance to be ready.
  *
- * \return The user's balance.
+ * \return The account balance as #CurrencyAmount.
  * \sa accountBalanceChanged()
  */
 CurrencyAmount Connection::accountBalance() const
@@ -1439,13 +1437,13 @@ CurrencyAmount Connection::accountBalance() const
  * This property cannot change after the connection has gone to state
  * ConnectionStatusConnected, so there is no change notification.
  *
- * This method requires Connection::FeatureCore to be enabled.
+ * This method requires Connection::FeatureCore to be ready.
  *
  * @return The capabilities of this connection.
  */
 ConnectionCapabilities Connection::capabilities() const
 {
-    if (!isReady()) {
+    if (!isReady(Connection::FeatureCore)) {
         warning() << "Connection::capabilities() used before connection "
             "FeatureCore is ready";
     }
@@ -1806,7 +1804,6 @@ void Connection::gotBalance(QDBusPendingCallWatcher *watcher)
  *
  * \return A pointer to the existing Client::ConnectionInterface object for this
  *         Connection object.
-
  */
 Client::ConnectionInterface *Connection::baseInterface() const
 {
@@ -1843,9 +1840,11 @@ PendingChannel *ConnectionLowlevel::createChannel(const QVariantMap &request)
  *                If timeout is -1, a default implementation-defined value that
  *                is suitable for inter-process communications (generally,
  *                25 seconds) will be used.
- * \return Pointer to a newly constructed PendingChannel object, tracking
- *         the progress of the request.
- * \sa PendingChannel
+ * \return A PendingChannel which will emit PendingChannel::finished
+ *         when the channel has been created, or an error occurred.
+ * \sa PendingChannel, ensureChannel(),
+ *     Account::createChannel(), Account::createAndHandleChannel(),
+ *     Account::ensureChannel(), Account::ensureAndHandleChannel()
  */
 PendingChannel *ConnectionLowlevel::createChannel(const QVariantMap &request,
         int timeout)
@@ -1913,9 +1912,11 @@ PendingChannel *ConnectionLowlevel::ensureChannel(const QVariantMap &request)
  *                If timeout is -1, a default implementation-defined value that
  *                is suitable for inter-process communications (generally,
  *                25 seconds) will be used.
- * \return Pointer to a newly constructed PendingChannel object, tracking
- *         the progress of the request.
- * \sa PendingChannel
+ * \return A PendingChannel which will emit PendingChannel::finished
+ *         when the channel is ensured to exist, or an error occurred.
+ * \sa PendingChannel, createChannel(),
+ *     Account::createChannel(), Account::createAndHandleChannel(),
+ *     Account::ensureChannel(), Account::ensureAndHandleChannel()
  */
 PendingChannel *ConnectionLowlevel::ensureChannel(const QVariantMap &request,
         int timeout)
@@ -1976,13 +1977,12 @@ PendingChannel *ConnectionLowlevel::ensureChannel(const QVariantMap &request,
  * this means that the PendingHandles object should not be used after the
  * Connection is destroyed.
  *
- * \sa PendingHandles
- *
  * \param handleType Type for the handles to request, as specified in
  *                   #HandleType.
  * \param names Names of the entities to request handles for.
- * \return Pointer to a newly constructed PendingHandles object, tracking
- *         the progress of the request.
+ * \return A PendingHandles which will emit PendingHandles::finished
+ *         when the handles have been requested, or an error occurred.
+ * \sa PendingHandles
  */
 PendingHandles *ConnectionLowlevel::requestHandles(HandleType handleType, const QStringList &names)
 {
@@ -2033,8 +2033,8 @@ PendingHandles *ConnectionLowlevel::requestHandles(HandleType handleType, const
  *
  * \param handleType Type of the handles given, as specified in #HandleType.
  * \param handles Handles to request a reference to.
- * \return Pointer to a newly constructed PendingHandles object, tracking
- *         the progress of the request.
+ * \return A PendingHandles which will emit PendingHandles::finished
+ *         when the handles have been referenced, or an error occurred.
  */
 PendingHandles *ConnectionLowlevel::referenceHandles(HandleType handleType, const UIntList &handles)
 {
@@ -2084,10 +2084,10 @@ PendingHandles *ConnectionLowlevel::referenceHandles(HandleType handleType, cons
  * finish with an error if a fatal error occurs during that process.
  *
  * \param requestedFeatures The features which should be enabled
- * \return A PendingReady object which will emit finished
- *         when the Connection has reached ConnectionStatusConnected, and initial setup
+ * \return A PendingReady which will emit PendingReady::finished
+ *         when the Connection has reached #ConnectionStatusConnected, and initial setup
  *         for basic functionality, plus the given features, has succeeded or
- *         failed
+ *         failed.
  */
 PendingReady *ConnectionLowlevel::requestConnect(const Features &requestedFeatures)
 {
@@ -2112,8 +2112,8 @@ PendingReady *ConnectionLowlevel::requestConnect(const Features &requestedFeatur
  * Account::setRequestedPresence() with Presence::offline() as an argument should generally be used
  * instead.
  *
- * \return A %PendingOperation, which will emit finished when the
- *         request finishes.
+ * \return A PendingOperation which will emit PendingOperation::finished
+ *         when the request has been made.
  */
 PendingOperation *ConnectionLowlevel::requestDisconnect()
 {
@@ -2149,19 +2149,19 @@ PendingOperation *ConnectionLowlevel::requestDisconnect()
  * #TP_QT4_ERROR_NOT_IMPLEMENTED.
  *
  * Similarly, if the connection isn't both connected and ready
- * (<code>status() == ConnectionStatusConnected && isReady()</code>), the returned
- * PendingContactAttributes instance will fail instantly with the
+ * (<code>status() == ConnectionStatusConnected && isReady(Connection::FeatureCore)</code>),
+ * the returned PendingContactAttributes instance will fail instantly with the
  * error #TP_QT4_ERROR_NOT_AVAILABLE.
  *
- * This method requires Connection::FeatureCore to be enabled.
+ * This method requires Connection::FeatureCore to be ready.
  *
  * \sa PendingContactAttributes
  *
  * \param handles A list of handles of type HandleTypeContact
  * \param interfaces D-Bus interfaces for which the client requires information
  * \param reference Whether the handles should additionally be referenced.
- * \return Pointer to a newly constructed PendingContactAttributes, tracking the
- *         progress of the request.
+ * \return A PendingContactAttributes which will emit PendingContactAttributes::fininshed
+ *         when the contact attributes have been retrieved, or an error occurred.
  */
 PendingContactAttributes *ConnectionLowlevel::contactAttributes(const UIntList &handles,
         const QStringList &interfaces, bool reference)
@@ -2181,7 +2181,7 @@ PendingContactAttributes *ConnectionLowlevel::contactAttributes(const UIntList &
     PendingContactAttributes *pending =
         new PendingContactAttributes(conn,
                 handles, interfaces, reference);
-    if (!conn->isReady()) {
+    if (!conn->isReady(Connection::FeatureCore)) {
         warning() << "ConnectionLowlevel::contactAttributes() used when not ready";
         pending->failImmediately(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
                 QLatin1String("The connection isn't ready"));
@@ -2294,7 +2294,7 @@ bool ConnectionLowlevel::hasImmortalHandles() const
  * The contact manager is responsible for all contact handling in this
  * connection, including adding, removing, authorizing, etc.
  *
- * \return Pointer to this connection ContactManager object.
+ * \return A pointer to the ContactManager object.
  */
 ContactManagerPtr Connection::contactManager() const
 {
@@ -2552,7 +2552,7 @@ QString ConnectionHelper::statusReasonToErrorName(Tp::ConnectionStatusReason rea
 /**
  * \fn void Connection::selfHandleChanged(uint newHandle)
  *
- * This signal is emitted when the value of selfHandle() changes.
+ * Emitted when the value of selfHandle() changes.
  *
  * \param newHandle The new connection self handle.
  * \sa selfHandle()
@@ -2561,7 +2561,7 @@ QString ConnectionHelper::statusReasonToErrorName(Tp::ConnectionStatusReason rea
 /**
  * \fn void Connection::selfContactChanged()
  *
- * This signal is emitted when the value of selfContact() changes.
+ * Emitted when the value of selfContact() changes.
  *
  * \sa selfContact()
  */
@@ -2569,7 +2569,7 @@ QString ConnectionHelper::statusReasonToErrorName(Tp::ConnectionStatusReason rea
 /**
  * \fn void Connection::accountBalanceChanged(const Tp::CurrencyAmount &accountBalance)
  *
- * This signal is emitted when the value of accountBalance() changes.
+ * Emitted when the value of accountBalance() changes.
  *
  * \param accountBalance The new user's balance of this connection.
  * \sa accountBalance()
