@@ -123,7 +123,10 @@ public:
 
     ContactPtr lookupContactByHandle(uint handle);
 
-    void requestContactAvatar(Contact *contact);
+    TELEPATHY_QT4_DEPRECATED void requestContactAvatar(Contact *contact);
+    void requestContactAvatars(const QList<ContactPtr> &contacts);
+
+    PendingOperation *refreshContactInfo(const QList<ContactPtr> &contact);
 
 Q_SIGNALS:
     void stateChanged(Tp::ContactListState state);
@@ -161,11 +164,14 @@ private Q_SLOTS:
     TELEPATHY_QT4_NO_EXPORT void onCapabilitiesChanged(const Tp::ContactCapabilitiesMap &);
     TELEPATHY_QT4_NO_EXPORT void onLocationUpdated(uint, const QVariantMap &);
     TELEPATHY_QT4_NO_EXPORT void onContactInfoChanged(uint, const Tp::ContactInfoFieldList &);
+    TELEPATHY_QT4_NO_EXPORT void doRefreshInfo();
 
 private:
+    class PendingRefreshContactInfo;
     class Roster;
     friend class Connection;
     friend class PendingContacts;
+    friend class PendingRefreshContactInfo;
     friend class Roster;
 
     TELEPATHY_QT4_NO_EXPORT ContactManager(Connection *parent);
@@ -183,6 +189,8 @@ private:
     TELEPATHY_QT4_NO_EXPORT PendingOperation *introspectRosterGroups();
     TELEPATHY_QT4_NO_EXPORT void resetRoster();
 
+    TELEPATHY_QT4_NO_EXPORT PendingOperation *refreshContactInfo(Contact *contact);
+
     struct Private;
     friend struct Private;
     Private *mPriv;
