@@ -107,7 +107,7 @@ void Contact::Private::updateAvatarData()
         return;
     }
 
-    parent->manager()->requestContactAvatar(parent);
+    parent->manager()->requestContactAvatars(QList<ContactPtr>() << ContactPtr(parent));
 }
 
 struct TELEPATHY_QT4_NO_EXPORT Contact::InfoFields::Private : public QSharedData
@@ -489,7 +489,7 @@ void Contact::requestAvatarData()
         return;
     }
 
-    manager()->requestContactAvatar(this);
+    return manager()->requestContactAvatars(QList<ContactPtr>() << ContactPtr(this));
 }
 
 /**
@@ -621,7 +621,8 @@ Contact::InfoFields Contact::infoFields() const
  */
 PendingOperation *Contact::refreshInfo()
 {
-   if (!mPriv->requestedFeatures.contains(FeatureInfo)) {
+    ConnectionPtr conn = manager()->connection();
+    if (!mPriv->requestedFeatures.contains(FeatureInfo)) {
         warning() << "Contact::refreshInfo() used on" << this
             << "for which FeatureInfo hasn't been requested - failing";
         return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_AVAILABLE),
@@ -630,19 +631,7 @@ PendingOperation *Contact::refreshInfo()
                 ContactPtr(this));
     }
 
-    ConnectionPtr connection = manager()->connection();
-    if (!connection->hasInterface(TP_QT4_IFACE_CONNECTION_INTERFACE_CONTACT_INFO)) {
-        return new PendingFailure(QLatin1String(TELEPATHY_ERROR_NOT_IMPLEMENTED),
-                QLatin1String("Connection does not support ContactInfo interface"),
-                ContactPtr(this));
-    }
-
-    Client::ConnectionInterfaceContactInfoInterface *contactInfoInterface =
-        connection->interface<Client::ConnectionInterfaceContactInfoInterface>();
-    return new PendingVoid(
-            contactInfoInterface->RefreshContactInfo(
-                UIntList() << mPriv->handle[0]),
-            ContactPtr(this));
+    return manager()->refreshContactInfo(QList<ContactPtr>() << ContactPtr(this));
 }
 
 /**
@@ -657,8 +646,7 @@ PendingOperation *Contact::refreshInfo()
  */
 PendingContactInfo *Contact::requestInfo()
 {
-    ContactPtr self = manager()->lookupContactByHandle(mPriv->handle[0]);
-    return new PendingContactInfo(self);
+    return new PendingContactInfo(ContactPtr(this));
 }
 
 /**
@@ -754,8 +742,7 @@ QString Contact::publishStateMessage() const
  */
 PendingOperation *Contact::requestPresenceSubscription(const QString &message)
 {
-    ContactPtr self = manager()->lookupContactByHandle(mPriv->handle[0]);
-    return manager()->requestPresenceSubscription(QList<ContactPtr >() << self, message);
+    return manager()->requestPresenceSubscription(QList<ContactPtr>() << ContactPtr(this), message);
 }
 
 /**
@@ -769,8 +756,7 @@ PendingOperation *Contact::requestPresenceSubscription(const QString &message)
  */
 PendingOperation *Contact::removePresenceSubscription(const QString &message)
 {
-    ContactPtr self = manager()->lookupContactByHandle(mPriv->handle[0]);
-    return manager()->removePresenceSubscription(QList<ContactPtr>() << self, message);
+    return manager()->removePresenceSubscription(QList<ContactPtr>() << ContactPtr(this), message);
 }
 
 /**
@@ -785,8 +771,7 @@ PendingOperation *Contact::removePresenceSubscription(const QString &message)
  */
 PendingOperation *Contact::authorizePresencePublication(const QString &message)
 {
-    ContactPtr self = manager()->lookupContactByHandle(mPriv->handle[0]);
-    return manager()->authorizePresencePublication(QList<ContactPtr>() << self, message);
+    return manager()->authorizePresencePublication(QList<ContactPtr>() << ContactPtr(this), message);
 }
 
 /**
@@ -800,8 +785,7 @@ PendingOperation *Contact::authorizePresencePublication(const QString &message)
  */
 PendingOperation *Contact::removePresencePublication(const QString &message)
 {
-    ContactPtr self = manager()->lookupContactByHandle(mPriv->handle[0]);
-    return manager()->removePresencePublication(QList<ContactPtr>() << self, message);
+    return manager()->removePresencePublication(QList<ContactPtr>() << ContactPtr(this), message);
 }
 
 /**
@@ -822,9 +806,8 @@ bool Contact::isBlocked() const
  */
 PendingOperation *Contact::block(bool value)
 {
-    ContactPtr self = manager()->lookupContactByHandle(mPriv->handle[0]);
-    return value ? manager()->blockContacts(QList<ContactPtr>() << self)
-                 : manager()->unblockContacts(QList<ContactPtr>() << self);
+    return value ? manager()->blockContacts(QList<ContactPtr>() << ContactPtr(this))
+                 : manager()->unblockContacts(QList<ContactPtr>() << ContactPtr(this));
 }
 
 /**
@@ -839,8 +822,7 @@ PendingOperation *Contact::block(bool value)
  */
 PendingOperation *Contact::block()
 {
-    ContactPtr self = manager()->lookupContactByHandle(mPriv->handle[0]);
-    return manager()->blockContacts(QList<ContactPtr>() << self);
+    return manager()->blockContacts(QList<ContactPtr>() << ContactPtr(this));
 }
 
 /**
@@ -858,8 +840,7 @@ PendingOperation *Contact::block()
  */
 PendingOperation *Contact::blockAndReportAbuse()
 {
-    ContactPtr self = manager()->lookupContactByHandle(mPriv->handle[0]);
-    return manager()->blockContactsAndReportAbuse(QList<ContactPtr>() << self);
+    return manager()->blockContactsAndReportAbuse(QList<ContactPtr>() << ContactPtr(this));
 }
 
 /**
@@ -873,8 +854,7 @@ PendingOperation *Contact::blockAndReportAbuse()
  */
 PendingOperation *Contact::unblock()
 {
-    ContactPtr self = manager()->lookupContactByHandle(mPriv->handle[0]);
-    return manager()->unblockContacts(QList<ContactPtr>() << self);
+    return manager()->unblockContacts(QList<ContactPtr>() << ContactPtr(this));
 }
 
 /**
@@ -907,8 +887,7 @@ QStringList Contact::groups() const
  */
 PendingOperation *Contact::addToGroup(const QString &group)
 {
-    ContactPtr self = manager()->lookupContactByHandle(mPriv->handle[0]);
-    return manager()->addContactsToGroup(group, QList<ContactPtr>() << self);
+    return manager()->addContactsToGroup(group, QList<ContactPtr>() << ContactPtr(this));
 }
 
 /**
@@ -925,8 +904,7 @@ PendingOperation *Contact::addToGroup(const QString &group)
  */
 PendingOperation *Contact::removeFromGroup(const QString &group)
 {
-    ContactPtr self = manager()->lookupContactByHandle(mPriv->handle[0]);
-    return manager()->removeContactsFromGroup(group, QList<ContactPtr>() << self);
+    return manager()->removeContactsFromGroup(group, QList<ContactPtr>() << ContactPtr(this));
 }
 
 void Contact::augment(const Features &requestedFeatures, const QVariantMap &attributes)
