@@ -1,7 +1,7 @@
 /**
  * This file is part of TelepathyQt4
  *
- * @copyright Copyright (C) 2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2010-2011 Collabora Ltd. <http://www.collabora.co.uk/>
  * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
@@ -25,44 +25,34 @@
 
 #include "TelepathyQt4/debug-internal.h"
 
+#include <TelepathyQt4/PendingVariantMap>
+
 namespace Tp
 {
 
 struct TELEPATHY_QT4_NO_EXPORT TubeChannel::Private
 {
     Private(TubeChannel *parent);
-    ~Private();
-
-    void init();
-
-    void extractTubeProperties(const QVariantMap &props);
 
     static void introspectTube(TubeChannel::Private *self);
 
-    ReadinessHelper *readinessHelper;
+    void extractTubeProperties(const QVariantMap &props);
 
+    // Public object
     TubeChannel *parent;
 
-    // Properties
+    ReadinessHelper *readinessHelper;
+
+    // Introspection
     TubeChannelState state;
-    TubeType type;
     QVariantMap parameters;
 };
 
 TubeChannel::Private::Private(TubeChannel *parent)
-    : parent(parent)
+    : parent(parent),
+      readinessHelper(parent->readinessHelper()),
+      state((TubeChannelState) -1)
 {
-}
-
-TubeChannel::Private::~Private()
-{
-}
-
-void TubeChannel::Private::init()
-{
-    // Initialize readinessHelper + introspectables here
-    readinessHelper = parent->readinessHelper();
-
     ReadinessHelper::Introspectables introspectables;
 
     ReadinessHelper::Introspectable introspectableTube(
@@ -71,45 +61,33 @@ void TubeChannel::Private::init()
             QStringList() << QLatin1String(TELEPATHY_INTERFACE_CHANNEL_INTERFACE_TUBE), // dependsOnInterfaces
             (ReadinessHelper::IntrospectFunc) &TubeChannel::Private::introspectTube,
             this);
-    introspectables[TubeChannel::FeatureTube] = introspectableTube;
+    introspectables[TubeChannel::FeatureCore] = introspectableTube;
 
     readinessHelper->addIntrospectables(introspectables);
 }
 
-void TubeChannel::Private::extractTubeProperties(const QVariantMap &props)
-{
-    state = (Tp::TubeChannelState)qdbus_cast<uint>(props[QLatin1String("State")]);
-    debug() << state << qdbus_cast<uint>(props[QLatin1String("State")]);
-    parameters = qdbus_cast<QVariantMap>(props[QLatin1String("Parameters")]);
-}
-
 void TubeChannel::Private::introspectTube(TubeChannel::Private *self)
 {
     TubeChannel *parent = self->parent;
 
-    debug() << "Introspect tube state";
-
-    Client::ChannelInterfaceTubeInterface *tubeInterface = 
+    debug() << "Introspecting tube properties";
+    Client::ChannelInterfaceTubeInterface *tubeInterface =
             parent->interface<Client::ChannelInterfaceTubeInterface>();
 
-    // It must be present
-    Q_ASSERT(tubeInterface);
+    parent->connect(tubeInterface,
+            SIGNAL(TubeChannelStateChanged(uint)),
+            SLOT(onTubeChannelStateChanged(uint)));
 
-    parent->connect(tubeInterface, SIGNAL(TubeChannelStateChanged(uint)),
-            parent, SLOT(onTubeChannelStateChanged(uint)));
+    PendingVariantMap *pvm = tubeInterface->requestAllProperties();
+    parent->connect(pvm,
+            SIGNAL(finished(Tp::PendingOperation *)),
+            SLOT(gotTubeProperties(Tp::PendingOperation *)));
+}
 
-    Client::DBus::PropertiesInterface *properties =
-            parent->interface<Client::DBus::PropertiesInterface>();
-
-    QDBusPendingCallWatcher *watcher =
-            new QDBusPendingCallWatcher(
-                    properties->GetAll(
-                            QLatin1String(TELEPATHY_INTERFACE_CHANNEL_INTERFACE_TUBE)),
-                    parent);
-    parent->connect(watcher,
-            SIGNAL(finished(QDBusPendingCallWatcher *)),
-            parent,
-            SLOT(gotTubeProperties(QDBusPendingCallWatcher *)));
+void TubeChannel::Private::extractTubeProperties(const QVariantMap &props)
+{
+    state = (Tp::TubeChannelState) qdbus_cast<uint>(props[QLatin1String("State")]);
+    parameters = qdbus_cast<QVariantMap>(props[QLatin1String("Parameters")]);
 }
 
 /**
@@ -117,19 +95,18 @@ void TubeChannel::Private::introspectTube(TubeChannel::Private *self)
  * \ingroup clientchannel
  * \headerfile TelepathyQt4/tube-channel.h <TelepathyQt4/TubeChannel>
  *
- * \brief This is the base class for all tube types.
+ * \brief The TubeChannel class is a base class for all tube types.
  *
  * A tube is a mechanism for arbitrary data transfer between two or more IM users,
  * used to allow applications on the users' systems to communicate without having
  * to establish network connections themselves.
  *
- * \note You should \b never create an abstract tube: you should use one of its
- * subclasses instead. At the moment, \c StreamTube and \c DBusTube are available.
+ * Note that TubeChannel should never be instantiated directly, instead one of its
+ * subclasses (e.g. IncomingStreamTubeChannel or OutgoingStreamTubeChannel) should be used.
  *
- * For more details, please refer to \telepathy_spec.
+ * See \ref async_model, \ref shared_ptr
  */
 
-// Features declaration and documentation
 /**
  * Feature representing the core that needs to become ready to make the
  * TubeChannel object usable.
@@ -138,16 +115,20 @@ void TubeChannel::Private::introspectTube(TubeChannel::Private *self)
  * TubeChannel methods.
  * See specific methods documentation for more details.
  */
-const Feature TubeChannel::FeatureTube =
-        Feature(QLatin1String(TubeChannel::staticMetaObject.className()), 0);
+const Feature TubeChannel::FeatureCore = Feature(QLatin1String(TubeChannel::staticMetaObject.className()), 0);
+
+/**
+ * \deprecated Use TubeChannel::FeatureCore instead.
+ */
+const Feature TubeChannel::FeatureTube = TubeChannel::FeatureCore;
 
 /**
  * Create a new TubeChannel channel.
  *
  * \param connection Connection owning this channel, and specifying the
  *                   service.
- * \param objectPath The object path of this channel.
- * \param immutableProperties The immutable properties of this channel.
+ * \param objectPath The channel object path.
+ * \param immutableProperties The channel immutable properties.
  * \return A TubeChannelPtr object pointing to the newly created
  *         TubeChannel object.
  */
@@ -163,8 +144,10 @@ TubeChannelPtr TubeChannel::create(const ConnectionPtr &connection,
  *
  * \param connection Connection owning this channel, and specifying the
  *                   service.
- * \param objectPath The object path of this channel.
- * \param immutableProperties The immutable properties of this channel.
+ * \param objectPath The channel object path.
+ * \param immutableProperties The channel immutable properties.
+ * \param coreFeature The core feature of the channel type, if any. The corresponding introspectable should
+ *                    depend on TubeChannel::FeatureCore.
  */
 TubeChannel::TubeChannel(const ConnectionPtr &connection,
         const QString &objectPath,
@@ -173,8 +156,6 @@ TubeChannel::TubeChannel(const ConnectionPtr &connection,
     : Channel(connection, objectPath, immutableProperties, coreFeature),
       mPriv(new Private(this))
 {
-    // Initialize
-    mPriv->init();
 }
 
 /**
@@ -186,20 +167,20 @@ TubeChannel::~TubeChannel()
 }
 
 /**
- * Returns the parameters associated with this tube, if any.
+ * Return the parameters associated with this tube, if any.
  *
- * This method requires TubeChannel::FeatureTube to be enabled.
+ * The parameters are populated when an outgoing tube is offered, but they are most useful in the
+ * receiving end, where the parameters passed to the offer can be extracted for the tube's entire
+ * lifetime to bootstrap legacy protocols. All parameters are passed unchanged.
  *
- * \return A dictionary of arbitrary parameters. Please refer to the spec for more details.
+ * This method requires TubeChannel::FeatureCore to be ready.
  *
- * \note For outgoing tubes, this function will return a valid value only after the tube has
- *       been offered successfully.
+ * \return The parameters as QVariantMap.
  */
 QVariantMap TubeChannel::parameters() const
 {
-    if (!isReady(FeatureTube)) {
-        warning() << "TubeChannel::parameters() used with "
-                "FeatureTube not ready";
+    if (!isReady(FeatureCore)) {
+        warning() << "TubeChannel::parameters() used with FeatureCore not ready";
         return QVariantMap();
     }
 
@@ -207,60 +188,94 @@ QVariantMap TubeChannel::parameters() const
 }
 
 /**
- * This methods returns the state of the tube in this channel. It can be used to find out
- * whether the tube is already opened.
+ * Return the state of this tube.
  *
- * \return The State of the tube in this channel.
+ * Change notification is via the stateChanged() signal.
  *
- * \note This method requires TubeChannel::FeatureTube to be enabled.
+ * This method requires TubeChannel::FeatureCore to be ready.
+ *
+ * \return The state as #TubeChannelState.
+ * \sa stateChanged()
  */
-TubeChannelState TubeChannel::tubeState() const
+TubeChannelState TubeChannel::state() const
 {
-    if (!isReady(FeatureTube)) {
-        warning() << "TubeChannel::tubeState() used with "
-                "FeatureTube not ready";
+    if (!isReady(FeatureCore)) {
+        warning() << "TubeChannel::state() used with FeatureCore not ready";
         return TubeChannelStateNotOffered;
     }
 
     return mPriv->state;
 }
 
+/**
+ * \deprecated Use state() instead.
+ */
+TubeChannelState TubeChannel::tubeState() const
+{
+    return state();
+}
+
 void TubeChannel::setParameters(const QVariantMap &parameters)
 {
     mPriv->parameters = parameters;
 }
 
-void TubeChannel::onTubeChannelStateChanged(uint newstate)
+void TubeChannel::onTubeChannelStateChanged(uint newState)
 {
-    mPriv->state = (Tp::TubeChannelState)newstate;
-    emit tubeStateChanged((Tp::TubeChannelState)newstate);
-    debug() << "staete changed to" << newstate;
+    if (newState == mPriv->state) {
+        return;
+    }
+
+    uint oldState = mPriv->state;
+
+    debug() << "Tube state changed to" << newState;
+    mPriv->state = (Tp::TubeChannelState) newState;
+
+    /* only emit stateChanged if we already received the state from initial introspection */
+    if (oldState != (uint) -1) {
+        emit stateChanged((Tp::TubeChannelState) newState);
+        // FIXME (API/ABI break) Remove tubeStateChanged call
+        emit tubeStateChanged((Tp::TubeChannelState) newState);
+    }
 }
 
-void TubeChannel::gotTubeProperties(QDBusPendingCallWatcher *watcher)
+void TubeChannel::gotTubeProperties(PendingOperation *op)
 {
-    QDBusPendingReply<QVariantMap> reply = *watcher;
+    if (!op->isError()) {
+        PendingVariantMap *pvm = qobject_cast<PendingVariantMap *>(op);
+
+        mPriv->extractTubeProperties(pvm->result());
 
-    if (!reply.isError()) {
-        QVariantMap props = reply.value();
-        mPriv->extractTubeProperties(props);
         debug() << "Got reply to Properties::GetAll(TubeChannel)";
-        mPriv->readinessHelper->setIntrospectCompleted(TubeChannel::FeatureTube, true);
-    }
-    else {
+        mPriv->readinessHelper->setIntrospectCompleted(TubeChannel::FeatureCore, true);
+    } else {
         warning().nospace() << "Properties::GetAll(TubeChannel) failed "
-                "with " << reply.error().name() << ": " << reply.error().message();
-        mPriv->readinessHelper->setIntrospectCompleted(TubeChannel::FeatureTube, false,
-                reply.error());
+            "with " << op->errorName() << ": " << op->errorMessage();
+        mPriv->readinessHelper->setIntrospectCompleted(TubeChannel::FeatureCore, false,
+                op->errorName(), op->errorMessage());
     }
 }
 
-// Signals documentation
+/**
+ * \fn void TubeChannel::stateChanged(Tp::TubeChannelState state)
+ *
+ * Emitted when the value of state() changes.
+ *
+ * \sa state The new state of this tube.
+ */
+
 /**
  * \fn void TubeChannel::tubeStateChanged(Tp::TubeChannelState state)
  *
- * Emitted when the state of the tube has changed, if
- * FeatureTube has been enabled.
+ * \deprecated Use stateChanged() instead.
  */
 
+void TubeChannel::connectNotify(const char *signalName)
+{
+    if (qstrcmp(signalName, SIGNAL(tubeStateChanged(Tp::TubeChannelState))) == 0) {
+        warning() << "Connecting to deprecated signal tubeStateChanged(Tp::TubeChannelState)";
+    }
+}
+
+
 } // Tp
