@@ -86,6 +86,8 @@ class SimpleTextObserver;
 class StreamedMediaChannel;
 class StreamedMediaStream;
 class StreamTubeChannel;
+class StreamTubeClient;
+class StreamTubeServer;
 class TextChannel;
 class TubeChannel;
 
@@ -143,6 +145,8 @@ typedef SharedPtr<SimpleTextObserver> SimpleTextObserverPtr;
 typedef SharedPtr<StreamedMediaChannel> StreamedMediaChannelPtr;
 typedef SharedPtr<StreamedMediaStream> StreamedMediaStreamPtr;
 typedef SharedPtr<StreamTubeChannel> StreamTubeChannelPtr;
+typedef SharedPtr<StreamTubeClient> StreamTubeClientPtr;
+typedef SharedPtr<StreamTubeServer> StreamTubeServerPtr;
 typedef SharedPtr<TextChannel> TextChannelPtr;
 typedef SharedPtr<TubeChannel> TubeChannelPtr;
 
