@@ -1,5 +1,5 @@
 /*
- * This file is part of TelepathyQt
+ * This file is part of TelepathyQt4
  *
  * Copyright (C) 2011 Collabora Ltd. <http://www.collabora.co.uk/>
  * Copyright (C) 2011 Nokia Corporation
@@ -23,26 +23,26 @@
 
 #include "pending-file-send.h"
 
-#include <TelepathyQt/Account>
-#include <TelepathyQt/AccountFactory>
-#include <TelepathyQt/AccountManager>
-#include <TelepathyQt/ChannelClassSpec>
-#include <TelepathyQt/ChannelClassSpecList>
-#include <TelepathyQt/ChannelFactory>
-#include <TelepathyQt/ClientRegistrar>
-#include <TelepathyQt/Connection>
-#include <TelepathyQt/ConnectionFactory>
-#include <TelepathyQt/Contact>
-#include <TelepathyQt/ContactCapabilities>
-#include <TelepathyQt/ContactFactory>
-#include <TelepathyQt/ContactManager>
-#include <TelepathyQt/Debug>
-#include <TelepathyQt/OutgoingFileTransferChannel>
-#include <TelepathyQt/PendingChannel>
-#include <TelepathyQt/PendingChannelRequest>
-#include <TelepathyQt/PendingContacts>
-#include <TelepathyQt/PendingOperation>
-#include <TelepathyQt/PendingReady>
+#include <TelepathyQt4/Account>
+#include <TelepathyQt4/AccountFactory>
+#include <TelepathyQt4/AccountManager>
+#include <TelepathyQt4/ChannelClassSpec>
+#include <TelepathyQt4/ChannelClassSpecList>
+#include <TelepathyQt4/ChannelFactory>
+#include <TelepathyQt4/ClientRegistrar>
+#include <TelepathyQt4/Connection>
+#include <TelepathyQt4/ConnectionFactory>
+#include <TelepathyQt4/Contact>
+#include <TelepathyQt4/ContactCapabilities>
+#include <TelepathyQt4/ContactFactory>
+#include <TelepathyQt4/ContactManager>
+#include <TelepathyQt4/Debug>
+#include <TelepathyQt4/OutgoingFileTransferChannel>
+#include <TelepathyQt4/PendingChannel>
+#include <TelepathyQt4/PendingChannelRequest>
+#include <TelepathyQt4/PendingContacts>
+#include <TelepathyQt4/PendingOperation>
+#include <TelepathyQt4/PendingReady>
 
 #include <QDebug>
 
@@ -91,11 +91,10 @@ void FileSender::onAMReady(PendingOperation *op)
 
     PendingReady *pr = qobject_cast<PendingReady*>(op);
     Q_ASSERT(pr != NULL);
-    Q_UNUSED(pr);
     qDebug() << "AccountManager ready";
 
-    mAccount = mAM->accountForObjectPath(
-            TP_QT_ACCOUNT_OBJECT_PATH_BASE + QLatin1Char('/') + mAccountName);
+    mAccount = mAM->accountForPath(
+            TP_QT4_ACCOUNT_OBJECT_PATH_BASE + QLatin1Char('/') + mAccountName);
     if (!mAccount) {
         qWarning() << "The account given does not exist";
         QCoreApplication::exit(1);
@@ -117,7 +116,6 @@ void FileSender::onAccountReady(PendingOperation *op)
 
     PendingReady *pr = qobject_cast<PendingReady*>(op);
     Q_ASSERT(pr != NULL);
-    Q_UNUSED(pr);
     qDebug() << "Account ready";
 
     qDebug() << "Checking if account is online...";
@@ -222,8 +220,8 @@ void FileSender::onTransferRequestFinished(PendingOperation *op)
     }
 
     // We should always receive outgoing channels of type FileTransfer, as requested,
-    // otherwise either MC or tp-qt itself is bogus, so let's assert in case they are
-    Q_ASSERT(chan->channelType() == TP_QT_IFACE_CHANNEL_TYPE_FILE_TRANSFER);
+    // otherwise either MC or tp-qt4 itself is bogus, so let's assert in case they are
+    Q_ASSERT(chan->channelType() == TP_QT4_IFACE_CHANNEL_TYPE_FILE_TRANSFER);
     Q_ASSERT(chan->isRequested());
 
     OutgoingFileTransferChannelPtr transferChannel = OutgoingFileTransferChannelPtr::qObjectCast(chan);
