@@ -1,13 +1,13 @@
 #include <QtTest/QtTest>
 
-#include <TelepathyQt/Constants>
-#include <TelepathyQt/Debug>
-#include <TelepathyQt/CapabilitiesBase>
-#include <TelepathyQt/ConnectionCapabilities>
-#include <TelepathyQt/ContactCapabilities>
-#include <TelepathyQt/Types>
+#include <TelepathyQt4/Constants>
+#include <TelepathyQt4/Debug>
+#include <TelepathyQt4/CapabilitiesBase>
+#include <TelepathyQt4/ConnectionCapabilities>
+#include <TelepathyQt4/ContactCapabilities>
+#include <TelepathyQt4/Types>
 
-#include <TelepathyQt/test-backdoors.h>
+#include <TelepathyQt4/test-backdoors.h>
 
 using namespace Tp;
 
@@ -53,6 +53,9 @@ void TestCapabilities::testConnCapabilities()
     QVERIFY(!connCaps.contactSearches());
     QVERIFY(!connCaps.contactSearchesWithSpecificServer());
     QVERIFY(!connCaps.contactSearchesWithLimit());
+    QVERIFY(!connCaps.contactSearch());
+    QVERIFY(!connCaps.contactSearchWithSpecificServer());
+    QVERIFY(!connCaps.contactSearchWithLimit());
     QVERIFY(!connCaps.streamTubes());
 
     RequestableChannelClassSpecList rccSpecs;
@@ -83,6 +86,9 @@ void TestCapabilities::testConnCapabilities()
     QVERIFY(!connCaps.contactSearches());
     QVERIFY(!connCaps.contactSearchesWithSpecificServer());
     QVERIFY(!connCaps.contactSearchesWithLimit());
+    QVERIFY(!connCaps.contactSearch());
+    QVERIFY(!connCaps.contactSearchWithSpecificServer());
+    QVERIFY(!connCaps.contactSearchWithLimit());
     QVERIFY(!connCaps.streamTubes());
     QCOMPARE(connCaps.allClassSpecs(), rccSpecs);
 
@@ -119,6 +125,9 @@ void TestCapabilities::testConnCapabilities()
     QVERIFY(connCaps.contactSearches());
     QVERIFY(connCaps.contactSearchesWithSpecificServer());
     QVERIFY(connCaps.contactSearchesWithLimit());
+    QVERIFY(connCaps.contactSearch());
+    QVERIFY(connCaps.contactSearchWithSpecificServer());
+    QVERIFY(connCaps.contactSearchWithLimit());
     QCOMPARE(connCaps.allClassSpecs(), rccSpecs);
 
     // start over
@@ -207,6 +216,9 @@ void TestCapabilities::testConnCapabilities()
     QVERIFY(!connCaps.contactSearches());
     QVERIFY(!connCaps.contactSearchesWithSpecificServer());
     QVERIFY(!connCaps.contactSearchesWithLimit());
+    QVERIFY(!connCaps.contactSearch());
+    QVERIFY(!connCaps.contactSearchWithSpecificServer());
+    QVERIFY(!connCaps.contactSearchWithLimit());
     QVERIFY(!connCaps.streamTubes());
 
     rccSpecs.append(RequestableChannelClassSpec::textChat());
@@ -286,6 +298,9 @@ void TestCapabilities::testConnCapabilities()
     QVERIFY(!connCaps.contactSearches());
     QVERIFY(!connCaps.contactSearchesWithSpecificServer());
     QVERIFY(!connCaps.contactSearchesWithLimit());
+    QVERIFY(!connCaps.contactSearch());
+    QVERIFY(!connCaps.contactSearchWithSpecificServer());
+    QVERIFY(!connCaps.contactSearchWithLimit());
     QVERIFY(!connCaps.streamTubes());
 }
 
